@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2008 Red Hat, Inc.
@@ -62,10 +61,20 @@
 #include "soup-input-stream.h"
 #include "soup-output-stream.h"
 
+#ifdef HAVE_AVAHI
+#include "gvfsdnssdutils.h"
+#include "gvfsdnssdresolver.h"
+#endif
+
 typedef struct _MountAuthData MountAuthData;
 
 static void mount_auth_info_free (MountAuthData *info);
 
+
+#ifdef HAVE_AVAHI
+static void dns_sd_resolver_changed  (GVfsDnsSdResolver *resolver, GVfsBackendDav *dav_backend);
+#endif
+
 typedef struct _AuthInfo {
 
    /* for server authentication */
@@ -92,6 +101,11 @@ struct _GVfsBackendDav
   GVfsBackendHttp parent_instance;
 
   MountAuthData auth_info;
+
+#ifdef HAVE_AVAHI
+  /* only set if we're handling a [dav|davs]+sd:// mounts */
+  GVfsDnsSdResolver *resolver;
+#endif
 };
 
 G_DEFINE_TYPE (GVfsBackendDav, g_vfs_backend_dav, G_VFS_TYPE_BACKEND_HTTP);
@@ -103,6 +117,14 @@ g_vfs_backend_dav_finalize (GObject *object)
 
   dav_backend = G_VFS_BACKEND_DAV (object);
 
+#ifdef HAVE_AVAHI
+  if (dav_backend->resolver != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (dav_backend->resolver, dns_sd_resolver_changed, dav_backend);
+      g_object_unref (dav_backend->resolver);
+    }
+#endif
+
   mount_auth_info_free (&(dav_backend->auth_info));
   
   if (G_OBJECT_CLASS (g_vfs_backend_dav_parent_class)->finalize)
@@ -210,10 +232,10 @@ path_equal (const char *a, const char *b, gboolean relax)
   a_len = strlen (a);
   b_len = strlen (b);
 
-  while (a[a_len - 1] == '/')
+  while (a_len > 0 && a[a_len - 1] == '/')
     a_len--;
 
-  while (b[b_len - 1] == '/')
+  while (b_len > 0 && b[b_len - 1] == '/')
     b_len--;
 
   if (a_len == b_len)
@@ -272,6 +294,9 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
     if (new_loc == NULL)
       return;
 
+    if (!SOUP_STATUS_IS_REDIRECTION(status))
+      return;
+
    new_uri = soup_uri_new_with_base (soup_message_get_uri (msg), new_loc);
    if (new_uri == NULL)
      {
@@ -283,6 +308,10 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 
    old_uri = soup_message_get_uri (msg);
 
+   /* copy over username and password to new_uri */
+   soup_uri_set_user(new_uri, old_uri->user);
+   soup_uri_set_password(new_uri, old_uri->password);
+
    /* Check if this is a trailing slash redirect (i.e. /a/b to /a/b/),
     * redirect it right away
     */
@@ -336,15 +365,13 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 #endif
          /* ELSE:
           *
-          * Three possibilities:
+          * Two possibilities:
           *
-          *   1) This was a non-3xx response that happened to
-          *      have a "Location" header
-          *   2) It's a non-redirecty 3xx response (300, 304,
+          *   1) It's a non-redirecty 3xx response (300, 304,
           *      305, 306)
-          *   3) It's some newly-defined 3xx response (308+)
+          *   2) It's some newly-defined 3xx response (308+)
           *
-          * We ignore all of these cases. In the first two,
+          * We ignore both of these cases. In the first,
           * redirecting would be explicitly wrong, and in the
           * last case, we have no clue if the 3xx response is
           * supposed to be redirecty or non-redirecty. Plus,
@@ -490,6 +517,36 @@ xml_node_iter_get_current (xmlNodeIter *iter)
   return iter->cur_node;
 }
 
+static gint
+http_to_gio_error(guint status_code)
+{
+  switch (status_code)
+    {
+      case SOUP_STATUS_NOT_FOUND:
+        return G_IO_ERROR_NOT_FOUND;
+        break;
+      case SOUP_STATUS_UNAUTHORIZED:
+      case SOUP_STATUS_PAYMENT_REQUIRED:
+      case SOUP_STATUS_FORBIDDEN:
+        return G_IO_ERROR_PERMISSION_DENIED;
+        break;
+      case SOUP_STATUS_REQUEST_TIMEOUT:
+        return G_IO_ERROR_TIMED_OUT;
+        break;
+      case SOUP_STATUS_CANT_RESOLVE:
+        return G_IO_ERROR_HOST_NOT_FOUND;
+        break;
+      case SOUP_STATUS_NOT_IMPLEMENTED:
+        return G_IO_ERROR_NOT_SUPPORTED;
+        break;
+      case SOUP_STATUS_INSUFFICIENT_STORAGE:
+        return G_IO_ERROR_NO_SPACE;
+        break;
+    }
+  
+  return G_IO_ERROR_FAILED;
+}
+
 static xmlDocPtr
 parse_xml (SoupMessage  *msg,
            xmlNodePtr   *root,
@@ -500,8 +557,7 @@ parse_xml (SoupMessage  *msg,
 
   if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
     {
-      
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+      g_set_error (error, G_IO_ERROR, http_to_gio_error (msg->status_code),
                    _("HTTP Error: %s"), msg->reason_phrase);
       return NULL;
     }
@@ -510,6 +566,7 @@ parse_xml (SoupMessage  *msg,
                        msg->response_body->length,
                        "response.xml",
                        NULL,
+                       XML_PARSE_NONET |
                        XML_PARSE_NOWARNING |
                        XML_PARSE_NOBLANKS |
                        XML_PARSE_NSCLEAN |
@@ -517,8 +574,8 @@ parse_xml (SoupMessage  *msg,
                        XML_PARSE_COMPACT);
   if (doc == NULL)
     { 
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "%s", _("Could not parse response"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+	                   _("Could not parse response"));
       return NULL;
     }
 
@@ -526,15 +583,17 @@ parse_xml (SoupMessage  *msg,
 
   if (*root == NULL || (*root)->children == NULL)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "%s", _("Empty response"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+	                   _("Empty response"));
+      xmlFreeDoc (doc);
       return NULL;
     }
 
   if (strcmp ((char *) (*root)->name, name))
     {
-        g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                     "%s", _("Unexpected reply from server"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                           _("Unexpected reply from server"));
+      xmlFreeDoc (doc);
       return NULL;
     }
 
@@ -848,6 +907,7 @@ ms_response_to_file_info (MsResponse *response,
           else if (node_has_name (node, "displayname"))
             {
               g_file_info_set_display_name (info, text);
+              have_display_name = TRUE;
             }
           else if (node_has_name (node, "getetag"))
             {
@@ -875,9 +935,16 @@ ms_response_to_file_info (MsResponse *response,
             }
           else if (node_has_name (node, "getlastmodified"))
             {
-              if (g_time_val_from_iso8601 (text, &tv))
-                g_file_info_set_modification_time (info, &tv);
-            }
+              SoupDate *sd;
+              GTimeVal tv;
+	      sd = soup_date_new_from_string(text);
+	      if (sd)
+	        {
+		  soup_date_to_timeval (sd, &tv);
+		  g_file_info_set_modification_time (info, &tv);
+		  soup_date_free (sd);
+		}
+	    }
         }
     }
 
@@ -1131,10 +1198,10 @@ stat_location (GVfsBackend  *backend,
 
   if (status != 207)
     {
-      g_set_error (error,
-                   G_IO_ERROR,
-                   http_error_code_from_status (status),
-                   msg->reason_phrase);
+      g_set_error_literal (error,
+	                   G_IO_ERROR,
+        	           http_error_code_from_status (status),
+                	   msg->reason_phrase);
 
       return FALSE;
     }
@@ -1142,9 +1209,9 @@ stat_location (GVfsBackend  *backend,
   res = stat_location_finish (msg, target_type, num_children);
 
   if (res == FALSE)
-    g_set_error (error, 
-                 G_IO_ERROR, G_IO_ERROR_FAILED,
-                 _("Response invalid"));
+    g_set_error_literal (error, 
+	                 G_IO_ERROR, G_IO_ERROR_FAILED,
+        	         _("Response invalid"));
 
   return res;
 }
@@ -1178,7 +1245,7 @@ soup_authenticate_from_data (SoupSession *session,
   MountAuthData *data;
   AuthInfo      *info;
 
-  g_print ("+ soup_authenticate_from_data (%s) \n",
+  g_debug ("+ soup_authenticate_from_data (%s) \n",
            retrying ? "retrying" : "first auth");
 
   if (retrying)
@@ -1214,7 +1281,7 @@ soup_authenticate_interactive (SoupSession *session,
   char              *new_password;
   char              *prompt;
 
-  g_print ("+ soup_authenticate_interactive (%s) \n",
+  g_debug ("+ soup_authenticate_interactive (%s) \n",
            retrying ? "retrying" : "first auth");
 
   data = (MountAuthData *) user_data;
@@ -1311,6 +1378,11 @@ soup_authenticate_interactive (SoupSession *session,
 
   if (res && !aborted)
     {
+      /* it's not safe to assume that we get the username filed in,
+         in the case that we provied a default username */
+      if (new_username == NULL)
+        new_username = g_strdup (info->username);
+
       soup_auth_authenticate (auth, new_username, new_password);
 
       g_free (info->username);
@@ -1322,7 +1394,7 @@ soup_authenticate_interactive (SoupSession *session,
   else
     soup_session_cancel_message (session, msg, SOUP_STATUS_CANCELLED);
 
-  g_print ("- soup_authenticate \n");
+  g_debug ("- soup_authenticate \n");
   g_free (prompt);
 }
 
@@ -1383,11 +1455,32 @@ g_mount_spec_to_dav_uri (GMountSpec *spec)
 }
 
 static GMountSpec *
-g_mount_spec_from_dav_uri (SoupURI *uri)
+g_mount_spec_from_dav_uri (GVfsBackendDav *dav_backend,
+                           SoupURI *uri)
 {
   GMountSpec *spec;
   const char *ssl;
 
+#ifdef HAVE_AVAHI
+  if (dav_backend->resolver != NULL)
+    {
+      const char *type;
+      const char *service_type;
+
+      service_type = g_vfs_dns_sd_resolver_get_service_type (dav_backend->resolver);
+      if (strcmp (service_type, "_webdavs._tcp") == 0)
+        type = "davs+sd";
+      else
+        type = "dav+sd";
+
+      spec = g_mount_spec_new (type);
+      g_mount_spec_set (spec,
+                        "host",
+                        g_vfs_dns_sd_resolver_get_encoded_triple (dav_backend->resolver));
+      return spec;
+    }
+#endif
+
   spec = g_mount_spec_new ("dav");
 
   g_mount_spec_set (spec, "host", uri->host);
@@ -1414,6 +1507,63 @@ g_mount_spec_from_dav_uri (SoupURI *uri)
   return spec;
 }
 
+#ifdef HAVE_AVAHI
+static SoupURI *
+dav_uri_from_dns_sd_resolver (GVfsBackendDav *dav_backend)
+{
+  SoupURI    *uri;
+  char       *user;
+  char       *path;
+  char       *address;
+  const char *service_type;
+  guint       port;
+
+  service_type = g_vfs_dns_sd_resolver_get_service_type (dav_backend->resolver);
+  address = g_vfs_dns_sd_resolver_get_address (dav_backend->resolver);
+  port = g_vfs_dns_sd_resolver_get_port (dav_backend->resolver);
+  user = g_vfs_dns_sd_resolver_lookup_txt_record (dav_backend->resolver, "u"); /* mandatory */
+  path = g_vfs_dns_sd_resolver_lookup_txt_record (dav_backend->resolver, "path"); /* optional */
+
+  /* TODO: According to http://www.dns-sd.org/ServiceTypes.html
+   * there's also a TXT record "p" for password. Handle this.
+   */
+
+  uri = soup_uri_new (NULL);
+
+  if (strcmp (service_type, "_webdavs._tcp") == 0)
+    soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTPS);
+  else
+    soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTP);
+
+  soup_uri_set_user (uri, user);
+
+  soup_uri_set_port (uri, port);
+
+  soup_uri_set_host (uri, address);
+
+  if (path != NULL)
+    soup_uri_set_path (uri, path);
+  else
+    soup_uri_set_path (uri, "/");
+
+
+  g_free (address);
+  g_free (user);
+  g_free (path);
+
+  return uri;
+}
+#endif
+
+#ifdef HAVE_AVAHI
+static void
+dns_sd_resolver_changed (GVfsDnsSdResolver *resolver,
+                         GVfsBackendDav    *dav_backend)
+{
+  /* TODO: handle when DNS-SD data changes */
+}
+#endif
+
 /* ************************************************************************* */
 /* Backend Functions */
 static void
@@ -1423,6 +1573,7 @@ do_mount (GVfsBackend  *backend,
           GMountSource *mount_source,
           gboolean      is_automount)
 {
+  GVfsBackendDav *dav_backend = G_VFS_BACKEND_DAV (backend);
   MountAuthData  *data;
   SoupSession    *session;
   SoupMessage    *msg_opts;
@@ -1435,10 +1586,43 @@ do_mount (GVfsBackend  *backend,
   gboolean        res;
   char           *last_good_path;
   char           *display_name;
+  const char     *host;
+  const char     *type;
 
-  g_print ("+ mount\n");
+  g_debug ("+ mount\n");
 
-  mount_base = g_mount_spec_to_dav_uri (mount_spec);
+  host = g_mount_spec_get (mount_spec, "host");
+  type = g_mount_spec_get (mount_spec, "type");
+
+#ifdef HAVE_AVAHI
+  /* resolve DNS-SD style URIs */
+  if ((strcmp (type, "dav+sd") == 0 || strcmp (type, "davs+sd") == 0) && host != NULL)
+    {
+      GError *error;
+
+      dav_backend->resolver = g_vfs_dns_sd_resolver_new_for_encoded_triple (host, "u");
+
+      error = NULL;
+      if (!g_vfs_dns_sd_resolver_resolve_sync (dav_backend->resolver,
+                                               NULL,
+                                               &error))
+        {
+          g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
+          g_error_free (error);
+          return;
+        }
+      g_signal_connect (dav_backend->resolver,
+                        "changed",
+                        (GCallback) dns_sd_resolver_changed,
+                        dav_backend);
+
+      mount_base = dav_uri_from_dns_sd_resolver (dav_backend);
+    }
+  else
+#endif
+    {
+      mount_base = g_mount_spec_to_dav_uri (mount_spec);
+    }
 
   if (mount_base == NULL)
     {
@@ -1537,12 +1721,17 @@ do_mount (GVfsBackend  *backend,
   mount_base->path = last_good_path;
 
   /* dup the mountspec, but only copy known fields */
-  mount_spec = g_mount_spec_from_dav_uri (mount_base);
+  mount_spec = g_mount_spec_from_dav_uri (dav_backend, mount_base);
 
   g_vfs_backend_set_mount_spec (backend, mount_spec);
   g_vfs_backend_set_icon_name (backend, "folder-remote");
   
-  display_name = g_strdup_printf (_("WebDAV on %s"), mount_base->host);
+#ifdef HAVE_AVAHI
+  if (dav_backend->resolver != NULL)
+    display_name = g_strdup (g_vfs_dns_sd_resolver_get_service_name (dav_backend->resolver));
+  else
+#endif
+    display_name = g_strdup_printf (_("WebDAV on %s"), mount_base->host);
   g_vfs_backend_set_display_name (backend, display_name);
   g_free (display_name);
   
@@ -1563,7 +1752,7 @@ do_mount (GVfsBackend  *backend,
                     data);
 
   g_vfs_job_succeeded (G_VFS_JOB (job));
-  g_print ("- mount\n");
+  g_debug ("- mount\n");
 }
 
 static PropName ls_propnames[] = {
@@ -1594,7 +1783,7 @@ do_query_info (GVfsBackend           *backend,
 
   error   = NULL;
 
-  g_print ("Query info %s\n", filename);
+  g_debug ("Query info %s\n", filename);
 
   msg = propfind_request_new (backend, filename, 0, ls_propnames);
 
@@ -1667,7 +1856,7 @@ do_enumerate (GVfsBackend           *backend,
  
   error = NULL;
 
-  g_print ("+ do_enumerate: %s\n", filename);
+  g_debug ("+ do_enumerate: %s\n", filename);
 
   msg = propfind_request_new (backend, filename, 1, ls_propnames);
 
@@ -1888,10 +2077,10 @@ write_ready (GObject      *source_object,
 
   if (nwrote < 0)
    {
-     g_vfs_job_failed (G_VFS_JOB (job),
-                       error->domain,
-                       error->code,
-                       error->message);
+     g_vfs_job_failed_literal (G_VFS_JOB (job),
+                              error->domain,
+                              error->code,
+                              error->message);
 
      g_error_free (error);
      return;
@@ -1941,10 +2130,10 @@ close_write_ready (GObject      *source_object,
                                       &error);
   if (res == FALSE)
     {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        error->domain,
-                        error->code,
-                        error->message);
+      g_vfs_job_failed_literal (G_VFS_JOB (job),
+                                error->domain,
+                                error->code,
+                                error->message);
 
       g_error_free (error);
     }
@@ -1993,9 +2182,9 @@ do_make_directory (GVfsBackend          *backend,
                         G_IO_ERROR_EXISTS,
                         _("Target file already exists"));
     else
-      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
-                        http_error_code_from_status (status),
-                        msg->reason_phrase);
+      g_vfs_job_failed_literal (G_VFS_JOB (job), G_IO_ERROR,
+                                http_error_code_from_status (status),
+                                msg->reason_phrase);
   else
     g_vfs_job_succeeded (G_VFS_JOB (job));
 
@@ -2040,10 +2229,10 @@ do_delete (GVfsBackend   *backend,
   status = g_vfs_backend_dav_send_message (backend, msg);
 
   if (!SOUP_STATUS_IS_SUCCESSFUL (status))
-    g_vfs_job_failed (G_VFS_JOB (job),
-                      G_IO_ERROR,
-                      http_error_code_from_status (status),
-                      msg->reason_phrase);
+    g_vfs_job_failed_literal (G_VFS_JOB (job),
+                              G_IO_ERROR,
+                              http_error_code_from_status (status),
+                              msg->reason_phrase);
   else
     g_vfs_job_succeeded (G_VFS_JOB (job));
 
@@ -2094,7 +2283,7 @@ do_set_display_name (GVfsBackend           *backend,
 
   if (SOUP_STATUS_IS_SUCCESSFUL (status))
     {
-      g_print ("new target_path: %s\n", target_path);
+      g_debug ("new target_path: %s\n", target_path);
       g_vfs_job_set_display_name_set_new_path (job, target_path);
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
