@@ -309,7 +309,7 @@ static void
 daemon_schedule_exit (GVfsDaemon *daemon)
 {
   if (daemon->exit_tag == 0)
-    daemon->exit_tag = g_timeout_add (1000, exit_at_idle, daemon);
+    daemon->exit_tag = g_timeout_add_seconds (1, exit_at_idle, daemon);
 }
 
 static void
@@ -354,7 +354,14 @@ g_vfs_daemon_re_register_job_sources (GVfsDaemon *daemon)
   for (l = daemon->job_sources; l != NULL; l = l->next)
     {
       if (G_VFS_IS_BACKEND (l->data))
-	g_vfs_backend_register_mount (l->data, NULL, NULL);
+	{
+	  GVfsBackend *backend = l->data;
+
+	  /* Only re-register if we registered before, not e.g
+	     if we're currently mounting. */
+	  if (g_vfs_backend_is_mounted (backend))
+	    g_vfs_backend_register_mount (backend, NULL, NULL);
+	}
     }
   
   g_mutex_unlock (daemon->lock);
@@ -364,7 +371,7 @@ void
 g_vfs_daemon_add_job_source (GVfsDaemon *daemon,
 			     GVfsJobSource *job_source)
 {
-  g_print ("Added new job source %p (%s)\n", job_source, g_type_name_from_instance ((gpointer)job_source));
+  g_debug ("Added new job source %p (%s)\n", job_source, g_type_name_from_instance ((gpointer)job_source));
   
   g_mutex_lock (daemon->lock);
 
@@ -439,7 +446,7 @@ void
 g_vfs_daemon_queue_job (GVfsDaemon *daemon,
 			GVfsJob *job)
 {
-  g_print ("Queued new job %p (%s)\n", job, g_type_name_from_instance ((gpointer)job));
+  g_debug ("Queued new job %p (%s)\n", job, g_type_name_from_instance ((gpointer)job));
   
   g_object_ref (job);
   g_signal_connect (job, "finished", (GCallback)job_finished_callback, daemon);
